@@ -22,17 +22,26 @@ It supports many filetypes, including:
     among others. See https://github.com/sk-/git-lint for the complete list.
 
 Usage:
-    git-lint [-f | --force] [FILENAME ...]
-    git-lint [-t | --tracked] [-f | --force]
+    git-lint [-f | --force] [--json] [--last-commit] [FILENAME ...]
+    git-lint [-t | --tracked] [-f | --force] [--json] [--last-commit]
     git-lint -h | --version
 
 Options:
-    -h            Show the usage patterns.
-    --version     Prints the version number.
-    -f --force    Shows all the lines with problems.
-    -t --tracked  Lints only tracked files.
+    -h             Show the usage patterns.
+    --version      Prints the version number.
+    -f --force     Shows all the lines with problems.
+    -t --tracked   Lints only tracked files.
+    --json         Prints the result as a json string. Useful to use it in
+                   conjunction with other tools.
+    --last-commit  Checks the last checked-out commit. This is mostly useful
+                   when used as: git checkout <revid>; git lint --last-commit.
 """
 
+from __future__ import unicode_literals
+
+import codecs
+import json
+import os
 import os.path
 import sys
 
@@ -94,15 +103,71 @@ def get_config():
     return linters.parse_yaml_config(yaml_config, repo_root)
 
 
-def main(argv):
+def format_comment(comment_data):
+    """Formats the data returned by the linters.
+
+    Given a dictionary with the fields: line, column, severity, message_id,
+    message, will generate a message like:
+
+    'line {line}, col {column}: {severity}: [{message_id}]: {message}'
+
+    Any of the fields may nbe absent.
+
+    Args:
+      comment_data: dictionary with the linter data.
+
+    Returns:
+      a string with the formatted message.
+    """
+    format_pieces = []
+    # Line and column information
+    if 'line' in comment_data:
+        format_pieces.append('line {line}')
+    if 'column' in comment_data:
+        if format_pieces:
+            format_pieces.append(', ')
+        format_pieces.append('col {column}')
+    if format_pieces:
+        format_pieces.append(': ')
+
+    # Severity and Id information
+    if 'severity' in comment_data:
+        format_pieces.append('{severity}: ')
+
+    if 'message_id' in comment_data:
+        format_pieces.append('[{message_id}]: ')
+
+    # The message
+    if 'message' in comment_data:
+        format_pieces.append('{message}')
+
+    return ''.join(format_pieces).format(**comment_data)
+
+
+def main(argv, stdout=sys.stdout, stderr=sys.stderr):
     """Main gitlint routine. To be called from scripts."""
+    # Wrap sys stdout for python 2, so print can understand unicode.
+    linesep = os.linesep
+    if sys.version_info[0] < 3:
+        if stdout == sys.stdout:
+            stdout = codecs.getwriter("utf-8")(stdout)
+        if stderr == sys.stderr:
+            stderr = codecs.getwriter("utf-8")(stderr)
+        linesep = unicode(os.linesep)
+
     arguments = docopt.docopt(__doc__,
                               argv=argv[1:],
                               version='git-lint v%s' % __VERSION__)
 
+    json_output = arguments['--json']
+
+    commit = None
+    if arguments['--last-commit']:
+        commit = git.last_commit()
+
     repository_root = git.repository_root()
     if repository_root is None:
-        sys.stderr.write('fatal: Not a git repository' + os.linesep)
+        stderr.write('fatal: Not a git repository' + os.linesep)
         return 128
 
     if arguments['FILENAME']:
@@ -110,12 +175,13 @@ def main(argv):
                                                    repository_root)
         if invalid_filenames:
             invalid_filenames.append(('', ''))
-            sys.stderr.write(
+            stderr.write(
                 os.linesep.join(invalid[1] for invalid in invalid_filenames))
             return 2
 
         changed_files = git.modified_files(repository_root,
-                                           tracked_only=arguments['--tracked'])
+                                           tracked_only=arguments['--tracked'],
+                                           commit=commit)
         modified_files = {}
         for filename in arguments['FILENAME']:
             normalized_filename = os.path.abspath(filename)
@@ -123,41 +189,64 @@ def main(argv):
                 normalized_filename)
     else:
         modified_files = git.modified_files(repository_root,
-                                            tracked_only=arguments['--tracked'])
+                                            tracked_only=arguments['--tracked'],
+                                            commit=commit)
 
     linter_not_found = False
     files_with_problems = 0
     gitlint_config = get_config()
+    json_result = {}
+
+    error = termcolor.colored('ERROR', 'red', attrs=('bold',))
+    skipped = termcolor.colored('SKIPPED', 'yellow', attrs=('bold',))
 
     for filename in sorted(modified_files.keys()):
         rel_filename = os.path.relpath(filename)
-        print('Linting file: %s' % termcolor.colored(rel_filename,
-                                                     attrs=('bold',)))
+        if not json_output:
+            stdout.write('Linting file: %s%s' %
+                         (termcolor.colored(rel_filename, attrs=('bold',)),
+                          linesep))
         if arguments['--force']:
             modified_lines = None
         else:
             modified_lines = git.modified_lines(filename,
-                                                modified_files[filename])
+                                                modified_files[filename],
+                                                commit=commit)
 
         result = linters.lint(
             filename, modified_lines, gitlint_config)
+        result = result[filename]
 
-        if result == 'OK':
-            result = termcolor.colored(result, 'green', attrs=('bold',))
-        elif result.startswith('SKIPPED:'):
-            result = result.replace(
-                'SKIPPED',
-                termcolor.colored('SKIPPED', 'yellow', attrs=('bold',)))
-        elif result.startswith('ERROR:'):
-            result = result.replace(
-                'ERROR',
-                termcolor.colored('ERROR', 'red', attrs=('bold',)))
+        output = ''
+        if result.get('error'):
+            output += os.linesep.join(
+                '%s: %s' % (error, reason) for reason in result.get('error')
+            )
             linter_not_found = True
+        if result.get('skipped'):
+            output += os.linesep.join(
+                '%s: %s' % (skipped, reason) for reason in result.get('skipped')
+            )
+        if result.get('comments', []) == []:
+            if not output:
+                output += termcolor.colored('OK', 'green', attrs=('bold',))
         else:
             files_with_problems += 1
+            output += os.linesep.join(format_comment(data)
+                                      for data in result['comments'])
 
-        print(result)
-        print('')
+        if json_output:
+            json_result[filename] = result
+        else:
+            stdout.write(output)
+            stdout.write(linesep + linesep)
+
+    if json_output:
+        # Hack to convert to unicode, Python3 returns unicode, wheres Python2
+        # returns str.
+        stdout.write(
+            json.dumps(json_result,
+                       ensure_ascii=False).encode('utf-8').decode('utf-8'))
 
     if files_with_problems > 0:
         return 1
