@@ -61,6 +61,14 @@ class UtilsTest(unittest.TestCase):
                                     r'(?P<line>\d+): .*|Debug: (?P<debug>.*)',
                                     groups=('line', 'debug'))))
 
+    def test_filter_lines_group_not_defined(self):
+        lines = ['1: foo', '12: bar', '', 'Debug: info']
+        self.assertEqual(
+            [('1', None), ('12', None)],
+            list(utils.filter_lines(lines,
+                                    r'(?P<line>\d+): .*',
+                                    groups=('line', 'debug'))))
+
     def test_open_for_write_file_exists(self):
         filename = 'foo/bar/new_file'
         with mock.patch('gitlint.utils.open',
@@ -93,15 +101,15 @@ class UtilsTest(unittest.TestCase):
                         side_effect=mock_abspath), \
              mock.patch('os.path.expanduser',
                         side_effect=lambda a: '/home/user'):
-            self.assertEquals(
+            self.assertEqual(
                 '/home/user/.git-lint/cache/linter1/foo/bar/file.txt',
                 utils._get_cache_filename('linter1', 'bar/file.txt'))
 
-            self.assertEquals(
+            self.assertEqual(
                 '/home/user/.git-lint/cache/linter2/foo/file.txt',
                 utils._get_cache_filename('linter2', 'file.txt'))
 
-            self.assertEquals(
+            self.assertEqual(
                 '/home/user/.git-lint/cache/linter3/bar/file.txt',
                 utils._get_cache_filename('linter3', '/bar/file.txt'))
 
@@ -144,6 +152,6 @@ class UtilsTest(unittest.TestCase):
              mock.patch('gitlint.utils.open',
                         mock.mock_open(read_data=content),
                         create=True) as mock_open:
-            self.assertEquals(
+            self.assertEqual(
                 content, utils.get_output_from_cache('linter', 'filename'))
             mock_open.assert_called_once_with(cache_filename)
